import { h } from "@stencil/core";
import { funnelDefs, colorFor, ColorPlugin, } from "../../utils";
import Between from "between.js";
Between._plugins.color = ColorPlugin;
let barHeight = [];
let barRefs = undefined;
const bgdFill = colorFor('white');
const xFill = colorFor('bada55');
let lastHex = '#bada55';
let tween = undefined;
let lastVal = [0, 0, 0, 0, 0];
let tween2 = [undefined, undefined, undefined, undefined, undefined];
const processColor = (next) => {
    if (!barRefs)
        return;
    const hex1 = lastHex;
    const hex2 = colorFor(next);
    // IMPORTANT:  we are tweening between the hex values, not the names...
    if (hex1 !== null && hex1 !== hex2) {
        if (tween) {
            tween.pause();
        }
        tween = new Between(hex1, hex2)
            .time(1000)
            .easing(Between.Easing.Cubic.InOut)
            .on('update', (v) => {
            barRefs.forEach(bar => {
                bar.setAttribute('fill', `${v}`);
            });
            lastHex = v;
        });
    }
};
const processValues = (next) => {
    if (!barRefs)
        return;
    barRefs.forEach((bar, index) => {
        const height = barHeight[index];
        const last = lastVal[index];
        const value = next[index];
        if (last !== null && last !== value) {
            if (tween2[index]) {
                tween2[index].pause();
            }
            // NOTE:  next & last are values between 0-100 (aka. percentage)
            tween2[index] = new Between(last, value)
                .time(1000)
                .easing(Between.Easing.Cubic.InOut)
                .on('update', (v) => {
                const y = height * ((100 - v) / 100);
                bar.setAttribute('y', `${y}`);
                lastVal[index] = v;
            });
        }
    });
};
const createFunnel = ({ id, height, width, x, y, drop }) => {
    const obj = {
        id: id,
        height,
        width,
        x,
        y,
        maskId: `mask-${id}`,
        maskUri: `url('#mask-${id}')`,
        mask: `0.5,0.5,0.5,${height - 0.5},${width - 0.5},${height - drop - 0.5},${width - 0.5},${drop + 0.5}`,
        matrix: `matrix(1,0,0,1,${x},${y})`,
        points: `0,0,0,${height},${width},${height - drop},${width},${drop}`,
    };
    return obj;
};
const generateBars = () => {
    return funnelDefs.map(item => {
        return createFunnel(item);
    });
};
const FunnelGfx = _props => {
    const bars = generateBars();
    const stroke = 'currentColor';
    // IMPORTANT:  initialize these arrays when you render...
    barHeight = Array(bars.length);
    barRefs = Array(bars.length);
    return (h("div", { class: "funnel" }, h("svg", { viewBox: "0 0 600 420" }, h("desc", null, "Created with Snap"), h("defs", null, bars.map(item => {
        return (h("mask", { id: item.maskId }, h("polygon", { points: item.mask, fill: bgdFill })));
    })), bars.map((item, index) => {
        const { height, width, maskUri, matrix, points } = item;
        barHeight[index] = height;
        return (h("g", { transform: matrix }, h("polygon", { points: points, fill: bgdFill, stroke: stroke }), h("rect", { ref: el => (barRefs[index] = el), x: "0", y: height, width: width, height: height, mask: maskUri, fill: xFill, stroke: stroke })));
    }))));
};
export { FunnelGfx, processColor, processValues };
export default FunnelGfx;
